package com.me.brickbuster.entity;

import com.badlogic.gdx.graphics.Pixmap;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.PolygonRegion;
import com.badlogic.gdx.graphics.g2d.PolygonSpriteBatch;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.graphics.glutils.ShapeRenderer;
import com.badlogic.gdx.math.EarClippingTriangulator;
import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.physics.box2d.Body;
import com.badlogic.gdx.physics.box2d.BodyDef;
import com.badlogic.gdx.physics.box2d.FixtureDef;
import com.badlogic.gdx.physics.box2d.PolygonShape;
import com.badlogic.gdx.utils.ShortArray;
import com.me.brickbuster.entity.powerup.PowerUpType;
import com.me.brickbuster.physics.CollisionListener;
import com.me.brickbuster.physics.EntityType;
import com.me.brickbuster.physics.PhysicsBody;
import com.me.brickbuster.state.FieldState;
import com.me.brickbuster.state.PlayState;

public class Brick extends Entity implements PhysicsBody, CollisionListener {

	public static final float BRICK_WIDTH = 5f;
	public static final float BRICK_HEIGHT = 2.5f;

	private static final EarClippingTriangulator ECT = new EarClippingTriangulator();
	private static final Vector2 tmp = new Vector2();

	private BrickType type;
	private BrickShape shape;
	private PowerUpType powerUpType;

	private Pixmap pm;
	private Texture solid;
	private TextureRegion region;

	private Body body;
	private boolean hitByBall = false;

	public Brick(FieldState state, BrickType type, BrickShape shape, PowerUpType powerUpType, float x, float y) {
		super(state, x, y);
		this.type = type;
		this.shape = shape;
		this.powerUpType = powerUpType;
		this.pm = new Pixmap(1,1, Pixmap.Format.RGBA8888);
		pm.setColor(type.getColor());
		pm.fill();
		this.solid = new Texture(pm);
		this.region = new TextureRegion(solid);
		createBody();
	}

	@Override
	public void render(ShapeRenderer sr) {
		PolygonShape shape = (PolygonShape) body.getFixtureList().get(0).getShape();
		float[] vertices = new float[shape.getVertexCount()*2];
		for (int i = 0; i < vertices.length/2; i++) {
			shape.getVertex(i, tmp);
			Vector2 vertex = body.getWorldPoint(tmp);
			vertices[i*2] = vertex.x * PlayState.PIXEL_PER_METER;
			vertices[i*2 + 1] = vertex.y * PlayState.PIXEL_PER_METER;
		}
		ShortArray triangleIndices = ECT.computeTriangles(vertices);

		PolygonRegion polyRegion = new PolygonRegion(region, vertices, triangleIndices.toArray());

		PolygonSpriteBatch pb = state.getGame().pb;
		pb.begin();
		pb.draw(polyRegion, 0, 0);
		pb.end();
	}

	@Override
	public void update(float dt) {
		if (hitByBall) {
			hit();
			hitByBall = false;
		}

		if (getY() - BRICK_HEIGHT < 0) {
			delete();
		}
	}

	@Override
	public Body getBody() {
		return body;
	}

	@Override
	public void createBody() {
		BodyDef brickBody = new BodyDef();
		brickBody.type = BodyDef.BodyType.StaticBody;
		brickBody.position.set(pos.cpy());

		PolygonShape brickShape = new PolygonShape();
		switch (shape) {
			case DIAMOND:
				brickShape.set(new float[] {
						-BRICK_WIDTH/2, 0,  // Left
						0, BRICK_HEIGHT/2,  // Up
						BRICK_WIDTH/2, 0,   // Right
						0, -BRICK_HEIGHT/2  // Down
				});
				break;
			case DOWN_RIGHT_TRIANGLE:
				brickShape.set(new float[] {
						BRICK_WIDTH/2, BRICK_HEIGHT/2,   // Top right
						BRICK_WIDTH/2, -BRICK_HEIGHT/2,  // Bottom right
						-BRICK_WIDTH/2, -BRICK_HEIGHT/2, // Bottom left
				});
				break;
			case UP_RIGHT_TRIANGLE:
				brickShape.set(new float[] {
						BRICK_WIDTH/2, BRICK_HEIGHT/2,   // Top right
						BRICK_WIDTH/2, -BRICK_HEIGHT/2,  // Bottom right
						-BRICK_WIDTH/2, BRICK_HEIGHT/2,  // Top left
				});
				break;
			case UP_LEFT_TRIANGLE:
				brickShape.set(new float[] {
						-BRICK_WIDTH/2, BRICK_HEIGHT/2,  // Top left
						BRICK_WIDTH/2, BRICK_HEIGHT/2,   // Top right
						-BRICK_WIDTH/2, -BRICK_HEIGHT/2, // Bottom left
				});
				break;
			case DOWN_LEFT_TRIANGLE:
				brickShape.set(new float[] {
						-BRICK_WIDTH/2, BRICK_HEIGHT/2,  // Top left
						BRICK_WIDTH/2, -BRICK_HEIGHT/2,  // Bottom right
						-BRICK_WIDTH/2, -BRICK_HEIGHT/2, // Bottom left
				});
				break;
			case HALF_LOWER:
				brickShape.set(new float[] {
						-BRICK_WIDTH/2, 0,               // Top left
						BRICK_WIDTH/2, 0,                // Top right
						BRICK_WIDTH/2, -BRICK_HEIGHT/2,  // Bottom right
						-BRICK_WIDTH/2, -BRICK_HEIGHT/2, // Bottom left
				});
				break;
			case HALF_UPPER:
				brickShape.set(new float[] {
						-BRICK_WIDTH/2, BRICK_HEIGHT/2,  // Top left
						BRICK_WIDTH/2, BRICK_HEIGHT/2,   // Top right
						BRICK_WIDTH/2, 0,                // Bottom right
						-BRICK_WIDTH/2, 0,               // Bottom left
				});
				break;
			case HALF_LEFT:
				brickShape.set(new float[] {
						-BRICK_WIDTH/2, BRICK_HEIGHT/2,  // Top left
						0, BRICK_HEIGHT/2,               // Top right
						0, -BRICK_HEIGHT/2,              // Bottom right
						-BRICK_WIDTH/2, -BRICK_HEIGHT/2, // Bottom left
				});
				break;
			case HALF_RIGHT:
				brickShape.set(new float[] {
						0, BRICK_HEIGHT/2,               // Top left
						BRICK_WIDTH/2, BRICK_HEIGHT/2,   // Top right
						BRICK_WIDTH/2, -BRICK_HEIGHT/2,  // Bottom right
						0, -BRICK_HEIGHT/2,              // Bottom left
				});
				break;
			default:
				brickShape.setAsBox(BRICK_WIDTH/2, BRICK_HEIGHT/2, Vector2.Zero, 0f);
		}

		FixtureDef brickFixture = new FixtureDef();
		brickFixture.shape = brickShape;
		brickFixture.friction = 0f;
		brickFixture.density = 0.5f;

		brickFixture.filter.categoryBits = EntityType.BRICK;
		brickFixture.filter.maskBits = EntityType.BALL | EntityType.BRICK | EntityType.BOUNDARY;


		body = state.world.createBody(brickBody);
		body.createFixture(brickFixture);
		body.setUserData(this);

		brickShape.dispose();
	}

	@Override
	public void beginContact(Entity contacted) {
		if (contacted instanceof Ball) {
			hitByBall = true;
		}
	}

	@Override
	public void endContact(Entity contacted) {
	}

	@Override
	public void dispose() {
		solid.dispose();
		pm.dispose();
	}

	public boolean hit() {
		if (((PlayState) state).bricks.size-1 <= Ball.BLOCKS_FOR_BOOST) {
			for (Ball ball : ((PlayState) state).balls) {
				ball.setSpeed(Ball.BOOST_SPEED);
			}
		}

		if (powerUpType != null) {
			((PlayState) state).powerUps.add(powerUpType.createInstance((PlayState) state, this));
		}

		deleted = true;
		return true;
	}

}
